$nomod51
$nolist
;**************************************************************************** 
; This is the disasembled source for V1.1 bootloader found in CH552 devices.
; When setting StartAddress to 3800h and leaving all other options at 0 it 
; produce exactly the same binary as stored in CH552 devices.
; However there are several options to make the code more compact so one
; can add extra functionality to the loader. Each option can be used without
; interfearing other options. 
; At max with serial code removed almost 400 bytes can be used for extra 
; functions         
;****************************************************************************
$include (..\inc\ch552_keil.h) ; keil a51 can use c header files

; <<< Use Configuration Wizard in Context Menu >>>
; <h> Configuration for WCH V1.1 Bootloader 

; <o> StartAddress <0-0x3800:0x800>                           
; <i> Startaddress for bootloader should be on a 0x800 boundary    
StartAddress       EQU  3800H  ;

; <o> CHANGE_BCD_VER <0=> off
;                    <1=> on
; <i> use USB bcdver field for reporting the bootloader version
CHANGE_BCD_VER     EQU  1

; <o> REPLACE_KEY_INIT <0=> off
;                      <1=> on
; <i> optimized code for the scramble key
REPLACE_KEY_INIT   EQU  1

; <o> ADD_USB_BUGFIX  <0=> off
;                     <1=> on
; <i> insert usb bugfix for Setup requests to clear possible STALL contitions
ADD_USB_BUGFIX     EQU  1

; <o> OPTIMIZE_CALLS     <0=> off
;                        <1=> on 
; <i> use ACALL/ AJMP instead of LCALL/LJMP a macro will do the job
OPTIMIZE_CALLS     EQU  1

; <o> REMOVE_SERIAL_CODE <0=> off
;                        <1=> on
; <i> disable serial mode, bootloader will work with on USB only
REMOVE_SERIAL_CODE EQU  0

; <o> USE_LOADER_EXT <0=> off
;                    <1=> on
; <i> include extra user dispatcher functions
USE_LOADER_EXT     EQU  1
; <<< end of configuration section >>>


;ROM_RESERVED       EQU  3FF0h
;LoaderOptions      EQU  3FF4h

;these macros select the short and long versions for CALL and JMP 
if (OPTIMIZE_CALLS=1)
  XCALL macro dest
    ACALL dest
  endm
  XJMP macro dest
    AJMP dest
  endm
else
  XCALL macro dest
    LCALL dest
  endm
  XJMP macro dest
    LJMP dest
  endm
endif

$list

dseg at 8
wValueLo:          ds 1
reqLen:            ds 1       ;wlengthLow
pCall:             ds 3       ;Call address for exec a program
bRequest:          ds 1
pDesc:             ds 3
scramble:          ds 4       ;Scramble Key
Addr:              ds 2
rLen:              ds 1
RAM_18:            ds 3       ;used as overlay
RAM_1B:            ds 3       ;
RAM_1E:            ds 1       ;

bseg at 0
bResponsValid:     dbit 1     ;1 if there is a valid reponse packet availble
bWriteProtected:   dbit 1     ;1 if CodeFlash is write protected 
bRunMode:          dbit 1     ;1 if in exec mode
if (REMOVE_SERIAL_CODE = 0)
bSerialMode:       dbit 1     ;1 if in serial mode
else
                   dbit 1     ;spare
endif
bRequestError:     dbit 1     ;1 if a usb request is not supported

dseg at 0x21
cmdbuffer:         ds 64      ;EP Buffer size
Stack:

xseg at 0
EP0_BUFFER:        ds  8      ; 0x0000 the control endpoint
                   ds  4      ; spare
EP2_OUT_BUFFER:    ds 64      ; 0x000C bulk out
EP2_IN_BUFFER:     ds 64      ; 0x004C bulk in
ExtraBuffer:       ds 64      ; 0x008C


cseg at StartAddress          ; bootloader start
    ljmp  C_C51STARTUP        ; maybe the core need a ljmp here
    nop                       ; so dont use a XJmp here

;=============== S U B R O U T I N E =======================================
Functiondispatcher:
;alias because this is overlay memory
__result    set RAM_18+0                ;dispatcher result
__Len       set RAM_18+2                ;response length 
__j         set RAM_18+1                ;loopvar  
__i         set R1                      ;another loopvar
__addr      set RAM_1B+0

    clr   A
    mov   __result, A                   ;result=0
    mov   __Len, A                      ;respLen=0
    Xcall ToggleProgMode
    mov   R0, #cmdbuffer+0              ;switch cmdbuffer[0]
    mov   A, @R0
    Xcall _cc_case

    dw    FunctionA2                    ;Chipdetect_v1
    db    0A2h
    dw    FunctionA5                    ;RunCommand_V1
    db    0A5h
    dw    FunctionA6                    ;SetScrambleKey_V1
    db    0A6h
    dw    FunctionA7                    ;VerifyCodeFlash_V1
    db    0A7h
    dw    FunctionA8                    ;WriteCodeFlash_V1
    db    0A8h
    dw    FunctionA9                    ;ErasePage_V1
    db    0A9h
if (USE_LOADER_EXT=1)
    dw    FunctionAC
    db    0ACh
endif
    dw    FunctionB5                    ;EraseDataFlash_V1 
    db    0B5h
    dw    FunctionB6                    ;WriteDataFlash_V1
    db    0B6h
    dw    FunctionB7                    ;ReadDataFlash_V1
    db    0B7h
    dw    FunctionB8                    ;WriteConfig_V1
    db    0B8h
    dw    FunctionB9                    ;ReadConfig_V1
    db    0B9h
    dw    FunctionBA                    ;WriteLoaderCfg_V1
    db    0BAh
    dw    FunctionBB                    ;GetLoaderVer_V1
    db    0BBh
    dw    0
    dw    DispatcherDefault

if (USE_LOADER_EXT=1)
;***************************************
;create the same function as in v2.31
;AC 04 55 AA <AddrLo> <AddrHi>
FunctionAC:
    mov   R0,#cmdbuffer+1

    mov   A,@R0 
    xrl   A,#0x04
    jnz   error
    
    inc   R0
    mov   A,@R0 
    xrl   A,#0x55
    jnz   error

    inc   R0
    mov   A,@R0 
    xrl   A,#0xAA
    jnz   error

    inc   R0
    mov   A,@R0            ;lsb first
    push  ACC  
    inc   R0
    mov   A,@R0            ;msb next
    push  ACC
    ret                    ;call the function
error:
    Xjmp  Dispatcher_error
endif    

;***************************************
;chipdetectV1
;<A2> <x> <"USB DBG CH559 & ISP">
FunctionA2:            
    mov   __result, CHIP_ID          ;
    clr   A
    mov   __i, A                        ;for (i=0,i<0x13;i++)
Loop1_A2:                               ;
    mov   A, __i
    mov   DPTR, #CopyRightMsg           ;  ->"USB DBG CH559 & ISP"
    movc  A, @A+DPTR
    mov   R7, A
    mov   A, #cmdbuffer+2               ;
    add   A, __i
    mov   R0, A
    mov   A, @R0
    xrl   A, R7
    jz    Next_A2
    Xjmp  Dispatcher_error              ;
Next_A2:
    inc   __i
    cjne  __i, #19, Loop1_A2
    Xjmp  Dispatcher_break
;***************************************
;verify_V1
;<A7> <len> <addrL> <addrH> [<data>]
FunctionA7:                             ;
    mov   R0, #cmdbuffer+3              ;
    mov   A, @R0
    mov   R6, A
    dec   R0
    mov   A, @R0
    mov   R5, A
    mov   A, R5
    mov   Addr+0, R6                    ;addr
    mov   Addr+1, A
    clr   A                             ;j=0
    mov   __j, A                        ;
Loop1_A7:                               ;do
    clr   A                             ;{
    mov   __i, A                        ;   i=0;
Loop2_A7:                               ;   {
    mov   A, #scramble                  ;
    add   A, __i
    mov   R0, A
    mov   A, @R0
    mov   R7, A                         ;       R7 = scramble[i];
    mov   A, #cmdbuffer+4               ;
    add   A, __j
    mov   R0, A
    mov   A, @R0
    xrl   A, R7
    mov   R7, A                         ;       R7 = scramble[i] ^ cmdbuffer[4+j];
    mov   DPL, Addr+1                   ;
    mov   DPH, Addr+0                   ;
    clr   A
    movc  A, @A+DPTR                    ;       A = CBYTE[addr]
    xrl   A, R7                         ;
    jz    Next_A7_J
    Xjmp  Dispatcher_error
Next_A7_j:
    inc   __j                            ;       j ++
    inc   Addr+1
    mov   A, Addr+1                     ;       addr++
    jnz   Next_A7_I
    inc   Addr+0
Next_A7_i:
    inc   __i                           ;       i++
    cjne  __i, #4, Loop2_A7             ;   }while (i<4)
    mov   A, __j
    clr   C
    mov   R0, #cmdbuffer+1              ;}while (j < cmdbuffer[1]);
    subb  A, @R0
    jc    Loop1_A7
    Xjmp  Dispatcher_break
;***************************************
;WriteCodeFlash_V1
;<a8> <len> <addrl> <addrH> [<datal>,<dataH>]
FunctionA8:                             ;
    jnb   bWriteProtected, AA_001
    Xjmp  DispatcherDone
AA_001:
    mov   R0, #cmdbuffer+3              ;FlAddr = Cmdbuffer[4] << 8 |  Cmdbuffer[3]
    mov   A, @R0
    mov   R6, A
    dec   R0
    mov   A, @R0
    mov   R5, A
    mov   A, R6
    mov   __addr+0, A
    mov   A, R5
    mov   __addr+1, A
    clr   A                             ;for (j=0;j<..;j++) 
    mov   __j, A
Loop1_AA:                               ;{
    clr   A                             ;   for (i=0;i<..;i++)
    mov   __i, A
Loop2_AA:                               ;   {
    mov   A, __i
    add   A, ACC                        ;      R7 = scramble[i<<1+1]
    add   A, #scramble+1
    mov   R0, A
    mov   A, @R0
    mov   R7, A
    mov   A, #cmdbuffer+5               ;      A = cmdbuffer[5+i]
    add   A, __j
    mov   R0, A
    mov   A, @R0
    xrl   A, R7                        ;      R6 = A ^ R7
    mov   R6, A
    mov   A, __i
    add   A, ACC                       ;      R5 = scramble[i<<1]
    add   A, #scramble+0
    mov   R0, A
    mov   A, @R0
    mov   R5, A
    mov   A, #cmdbuffer+4              ;      A = cmdbuffer[4+i]
    add   A, __j
    mov   R0, A
    mov   A, @R0
    xrl   A, R5                        ;      R4 = R5 ^ A
    mov   R5, A
    mov   A, R6
    mov   R4, A
    mov   R6, #0
    mov   A, __addr+1                  ;      dispresult=FlashCodeEntry(addr+j,R4/R5)
    add   A, __j
    mov   R7, A
    mov   A, R6
    addc  A, __addr+0
    mov   R6, A
    Xcall FlashCodeEntry
    mov   __result, R7
    mov   A, __result                  ;      if (dispresult) break;
    jz    Next_AA_j
    Xjmp  Dispatcher_break             ;
Next_AA_j:
    inc   __j                          ;      j+=2
    inc   __j
    inc   __i                          ;      i++
    cjne  __i, #2, Loop2_AA
    mov   A, __j
    clr   C
    mov   R0, #cmdbuffer+1             ; 
    subb  A, @R0
    jc    Loop1_AA
    Xjmp  Dispatcher_break
;**************************************
;erase_page_cmd_v1
;<A9> <len> <pageL> <pageh>
FunctionA9:                ;
    mov   R0, #cmdbuffer+3              ;FlAddr = Cmdbuffer[4] << 8 |  Cmdbuffer[3]
    mov   A,  @R0
    mov   R6, A
    dec   R0
    mov   A,  @R0
    mov   R5, A
    mov   A,  R6
    mov   __addr+0, A
    mov   A, R5
    mov   __addr+1, A
    jb    bWriteProtected, A9_001      ;if (!bWriteProtected) break;
    Xjmp  Dispatcher_break
A9_001:
    orl   A, __addr+0                  ;if (__addr)
    jz    A9_Loop
Dispatcher_error:                      ;{
    mov   __result, # -1               ;   dispresult = -1; break;  
    Xjmp  Dispatcher_break             ;}
A9_Loop:                               ;do
    mov   A, #0FFh
    mov   R5, A
    mov   R4, A
    mov   R7, __addr+1
    mov   R6, __addr+0
    Xcall FlashCodeEntry               ;   FlashCodeEntry(__addr,0xFFFF)   
    mov   __result, R7
    mov   A, __result
    jz    A9_Next_FlAddr
    Xjmp  Dispatcher_break
A9_Next_FlAddr:
    mov   A, #2                        ;   FlAddr+=2;
    add   A, __addr+1
    mov   __addr+1, A
    clr   A
    addc  A, __addr+0
    mov   __addr+0, A
    anl   A, #3
    orl   A, __addr+1
    jnz   A9_Loop                      ;} while(FlAddr & 0x3FF) 
    clr   bWriteProtected
    Xjmp  Dispatcher_break
;***************************************
;setkeyV1 new shorter
;<A6> <len> <key0> <key1> <key2> <key3>
FunctionA6:                            ;
if (REPLACE_KEY_INIT = 0)   
    mov   R0, #cmdbuffer+2
    mov   A, @R0
    mov   scramble+0, A
    inc   R0
    mov   A, @R0
    mov   scramble+1, A
    inc   R0
    mov   A, @R0
    mov   scramble+2, A
    inc   R0
    mov   A, @R0
    mov   scramble+3, A
    Xjmp  Dispatcher_break    
else
    mov   scramble+0, cmdbuffer+2
    mov   scramble+1, cmdbuffer+3
    mov   scramble+2, cmdbuffer+4
    mov   scramble+3, cmdbuffer+5
    Xjmp  Dispatcher_break 
endif
;***************************************
;write 2 config bytes
;<b8> <> <cfgl> <cfgh> <>
FunctionB8:                             ;
    mov   DPTR, # (ROM_CFG_ADDR)
    clr   A
    movc  A, @A+DPTR
    mov   R0, #cmdbuffer+2              ;
    xrl   A, @R0
    jnz   code_3972
    inc   DPTR
    movc  A, @A+DPTR
    inc   R0
    xrl   A, @R0
    jnz   code_3972
    Xjmp  Dispatcher_break
code_3972:                              ;
    mov   R0, #cmdbuffer+3              ; 
    sjmp  code_39C4
;***************************************
;PrepareIap
;  0    1    2   3     4      5     6   7   8   9
;<ba> <len> <0x57> <0xA8?> <runl> <runh> [<?> <?> <?> <?>]
FunctionBA:                             ;
    mov   DPTR, #(ROM_CFG_ADDR-2)       ;if  (cmdbuffer[4] != CBYTE[0x3FF6])
    clr   A
    movc  A, @A+DPTR
    mov   R0, #cmdbuffer+4              ;
    xrl   A, @R0
    jnz   BA_001                        ;  &&(cmdbuffer[5] != CBYTE[0x3FF7])
    inc   DPTR
    movc  A, @A+DPTR
    inc   R0
    xrl   A, @R0
    jz    BA_003
BA_001:                                 ;{
    mov   R0, #cmdbuffer+3              ;   FlashCode(LoaderOptions,&cmdbuffer[3])
    mov   R7, #low (ROM_CFG_ADDR-4)     ;
    Xcall FlashCode
    mov   __result, R7
    mov   A, __result
    jz    BA_002
    Xjmp  Dispatcher_break
BA_002:
    mov   R0, #cmdbuffer+5              ;  
    mov   R7, #low (ROM_CFG_ADDR-2)     ;   FlashCode(LoaderOptions+2,&cmdbuffer[5])
    Xcall FlashCode
    mov   __result, R7
    mov   A, __result
    jz    BA_003
    Xjmp  Dispatcher_break              ;}
BA_003:                                 ;
    mov   R0, #cmdbuffer+1              ;if (cmdbuffer[1]==8) 
    mov   A, @R0
    xrl   A, #8
    jz    BA_004
    Xjmp  Dispatcher_break
BA_004:
    mov   DPTR, # (ROM_CFG_ADDR)        ;
    movc  A, @A+DPTR
    mov   R0, #cmdbuffer+8              ;
    xrl   A, @R0
    jnz   code_39C2
    inc   DPTR
    movc  A, @A+DPTR
    inc   R0
    xrl   A, @R0
    jnz   code_39C2
    Xjmp  Dispatcher_break
code_39C2:                              ;
    mov   R0, #cmdbuffer+9              ;

code_39C4:                              ;
    mov   A, @R0
    mov   R6, A
    dec   R0
    mov   A, @R0
    mov   R5, A
    mov   A, R6
    mov   R4, A
    mov   R7, # low  (ROM_CFG_ADDR)     ;
    mov   R6, # high (ROM_CFG_ADDR)     ;
    Xcall FlashCodeEntry
    mov   __result, R7
    Xjmp  Dispatcher_break
;***************************************
;read config bytes and serial no 
;<b9> <> <>
FunctionB9:                             ;
    mov   Addr+0, #high (ROM_CFG_ADDR)  ;
    mov   Addr+1, #low  (ROM_CFG_ADDR)  ;
loop_B9:                                ;while (repLen < 8)   
    mov   A, __Len
    clr   C
    subb  A, #8
    jc    code_39E7
    Xjmp  Dispatcher_break
code_39E7:                              ;
    mov   A, __Len
    add   A, #low  (ROM_CFG_ADDR)       ;R7=CBYTE[ROM_CFG_ADDR+len]
    mov   R7, A
    clr   A
    addc  A, #high (ROM_CFG_ADDR)       ;
    mov   DPL, R7                       ;
    mov   DPH, A                        ;
    clr   A
    movc  A, @A+DPTR                    ; 
    mov   R7, A
    mov   A, #low EP2_IN_BUFFER         ;EP2_IN_BUFFER[Len] = CBYTE[ROM_CFG_ADDR+len]
    add   A, __Len
    mov   DPL, A                        ;
    clr   A
    addc  A, #high EP2_IN_BUFFER
    mov   DPH, A                        ;
    mov   A, R7
    movx  @DPTR, A
    inc   __Len                     ;
    sjmp  loop_B9
;***************************************
;get loader version
;<bb>
FunctionBB:       ;
    mov   __result, #011h
    Xjmp  Dispatcher_break
;***************************************
;ReadDataFlash_V1
FunctionB7:                             ;
    mov   ROM_ADDR_H, #high DATA_FLASH_ADDR ;
code_3A10:                              ;
    mov   A, __Len
    clr   C
    subb  A, #080h                      ;
    jnc   code_3A32                     
    mov   A, __Len                 
    add   A, ACC                        ;
    mov   ROM_ADDR_L, A                 ; 
    mov   ROM_CTRL, #ROM_CMD_READ       ;
    mov   A, # low  EP2_IN_BUFFER       ;
    add   A, __Len                 
    mov   DPL, A                        ;
    clr   A                             
    addc  A, # high EP2_IN_BUFFER            
    mov   DPH, A                        ;
    mov   A, ROM_DATA_L                 ;
    movx  @DPTR, A                      
    inc   __Len                
    sjmp  code_3A10                     
code_3A32:                              ;
    setb  bResponsValid
    sjmp  Dispatcher_break
;***************************************
;write_dataflashV1
;<B6> <len>, <addrL> <addrh> <data...>
FunctionB6:       ;
    mov   R0, #cmdbuffer+2 ;
    mov   A, @R0
    mov   __addr+0, #0
    mov   __addr+1, A
    mov   ROM_ADDR_H, #high DATA_FLASH_ADDR ;
    clr   A
    mov   __j, A
loop1_b6:
    clr   A
    mov   __i, A
loop2_b6:
    mov   A, __j
    add   A, __addr+1
    mov   R7, A
    mov   A, #scramble+0
    add   A, __i
    mov   R0, A
    mov   A, @R0
    mov   R6, A
    mov   A, #cmdbuffer+4
    add   A, __j
    mov   R0, A
    mov   A, @R0
    xrl   A, R6
    mov   R5, A
    Xcall FlashData
    mov   __result, R7
    mov   A, __result
    jnz   Dispatcher_break
    inc   __j
    inc   __i
    cjne  __i, #4, loop2_b6
    mov   A, __j
    clr   C
    mov   R0, #cmdbuffer+1 ;
    subb  A, @R0
    jc    loop1_b6
    sjmp  Dispatcher_break
;***************************************
;EraseDataFlash_V1
FunctionB5:       ;
    mov   R0, #cmdbuffer+2 ;
    mov   A, @R0
    mov   __i, A
    mov   ROM_ADDR_H, # high DATA_FLASH_ADDR ; 
code_3A79:        ;
    mov   A, __i
    clr   C
    subb  A, #080h ;
    jnc   Dispatcher_break
    xch   A, R7
    mov   A, __i
    xch   A, R7
    mov   R5, #0FFh
    Xcall FlashData
    mov   __result, R7
    mov   A, __result
    jnz   Dispatcher_break
    inc   __i
    sjmp  code_3A79

;***************************************
;run_cmd_v1
;<A5> <x> <[0x00 | 0x01] <0x00>
FunctionA5:       ;
    mov   R0, #cmdbuffer+3 
    mov   A, @R0
    dec   R0
    orl   A, @R0
    jz    Dispatcher_break
    setb  bRunMode
    sjmp  Dispatcher_break
;***************************************
DispatcherDefault:                      ;
    mov   __result, #-2                 ;
;***************************************
Dispatcher_break:                       ;
    Xcall ToggleProgMode
if (REMOVE_SERIAL_CODE = 0)
    jb    bSerialMode, code_3ADC 
endif
    mov   A,__Len                       ;if (Len)
    jz    code_3AB9                     ;
    setb  C
    subb  A, #040h                      ;  result = (responseLen > 0x40) ? 0x40 : responseLen
    jc    code_3AB1
    mov   R7, #040h                     ;
    sjmp  code_3AB3
code_3AB1:                              ;
    mov   R7, __Len
code_3AB3:                              ;
    mov   UEP2_T_LEN, R7                ;   UEP2_T_LEN= result;
    anl   UEP2_CTRL, #not MASK_UEP_T_RES;
    ret
code_3AB9:                              ;
    mov   DPTR, #EP2_IN_BUFFER          ; 
    mov   A, __result
    movx  @DPTR, A
    clr   A
    inc   DPTR
    movx  @DPTR, A
    mov   UEP2_T_LEN, #2                ;
    anl   UEP2_CTRL, #not MASK_UEP_T_RES;
    jnb   bRunMode, DispatcherDone
    mov   __result, # 50                ;50 loops 
code_3ACE:                              ;
    mov   R7, __result
    dec   __result
    mov   A, R7
    jz    DispatcherDone
    mov   R7, # 100                     ;
    Xcall Shortdelay
    sjmp  code_3ACE
if (REMOVE_SERIAL_CODE = 0)
code_3ADC:                              ;
    mov   A, __Len
    jz    code_3AFA
    clr   A
    mov   __i, A
code_3AE2:                              ;
    mov   A, __i
    clr   C
    subb  A, __Len
    jnc   DispatcherDone
    mov   A, #low  EP2_IN_BUFFER        ;
    add   A, __i
    mov   DPL, A                        ;
    clr   A
    addc  A, #high EP2_IN_BUFFER
    mov   DPH, A                        ;
    movx  A, @DPTR
    mov   R7, A
    Xcall Serial1PutChar                ;
    inc   __i
    sjmp  code_3AE2

code_3AFA:                              ;
    mov   R7, __result
    Xcall Serial1PutChar
    clr   A
    mov   R7, A
    Xcall Serial1PutChar
endif
DispatcherDone:                         ;B04
    ret

; =============== S U B R O U T I N E =======================================
HandleUsbEvents:                        ;
    jb    UIF_TRANSFER, code_3B0B       ;
    Xjmp  UsbReset
code_3B0B:                              ;
    mov   A, USB_INT_ST                 ;
    anl   A, #bUIS_TOKEN1 or bUIS_TOKEN0 or 0x0F;
    jnz   code_3B14
    Xjmp  EP0OutTransfer
code_3B14:                              ;
    add   A, #0E0h                      ; 
    jnz   code_3B1B
    Xjmp  EP0InTransfer
code_3B1B:                              ;
    add   A, #0FEh                      ; 
    jz    EP2InTransfer
    add   A, #0F2h                      ; 
    jz    EP0Setup
    add   A, #02Eh                      ; 
    jz    EP2OutTransfer
    Xjmp  break

;copy the bulkpacket to the cmdbuffer
EP2OutTransfer:                         ;
    jb    U_TOG_OK, code_3B30           ;
    Xjmp  break
code_3B30:                              ;
    mov   rLen, USB_RX_LEN              ;
    mov   RAM_1B+0, #1
    mov   RAM_1B+1, #high EP2_OUT_BUFFER
    mov   RAM_1B+2, #low  EP2_OUT_BUFFER
    mov   RAM_1E, rLen
    mov   R3, #0
    mov   R2, #0
    mov   R1, #cmdbuffer+0              ; 
    Xcall _memcpy
if (REMOVE_SERIAL_CODE = 0)
    clr   bSerialMode
endif
    Xcall Functiondispatcher
    Xjmp  break
;return the response back to the host
EP2InTransfer:                          ;
    jnb   bResponsValid, code_3B73
    clr   bResponsValid
    mov   RAM_1B+0, #1                  ;
    mov   RAM_1B+1, #high ExtraBuffer
    mov   RAM_1B+2, #low  ExtraBuffer   ;
    mov   RAM_1E,   # 40h               ;
    mov   R3, #1                        ;xdata
    mov   R2, #high EP2_IN_BUFFER
    mov   R1, #low  EP2_IN_BUFFER       ; 
    Xcall _memcpy
    mov   UEP2_T_LEN, # 64              ; 
    anl   UEP2_CTRL, #not MASK_UEP_T_RES; 0xFC
    Xjmp  break

code_3B73:                              ;
    mov   A, UEP2_CTRL                  ;
    anl   A, #not MASK_UEP_T_RES        ;0xFC
    orl   A, #UEP_T_RES_NAK
    mov   UEP2_CTRL, A                  ;
    Xjmp  break

EP0Setup:                               ;
if (ADD_USB_BUGFIX = 1)
    ANL   UEP0_CTRL,#0F2H               ;bugfix todo
endif
    mov   A, USB_RX_LEN                 ;
    xrl   A, #8
    jnz   code_3BF3
    mov   DPTR, # EP0_BUFFER+6
    movx  A, @DPTR
    mov   reqLen, A
    clr   bRequestError
    mov   DPTR, # EP0_BUFFER+0
    movx  A, @DPTR
    anl   A, #060h                      ;
    jz    code_3B98
    setb  bRequestError
    sjmp  Setup_break

code_3B98:                              ;
    mov   DPTR, # EP0_BUFFER+1
    movx  A, @DPTR
    mov   bRequest, A
    mov   A, bRequest
    add   A, #0FBh                      ; 
    jz    Set_Address
    add   A, #0FDh                      ;
    jz    Get_Config
    dec   A
    jz    Set_Config
    add   A, #3
    jnz   Setup_default
    mov   DPTR, # EP0_BUFFER+3
    movx  A, @DPTR
    add   A, #0FEh                      ;
    jz    GetDescConfig
    inc   A
    jnz   GetDescUnknown
GetDescDevice:
    mov   pDesc+0, #0FFh
    mov   pDesc+1, #high DeviceDesc
    mov   pDesc+2, #low  DeviceDesc
    mov   rLen,    #18                  ;sizeof(DeviceDesc)
    sjmp  Setup_break
GetDescConfig:
    mov   pDesc+0, #0FFh
    mov   pDesc+1, #high ConfigDesc
    mov   pDesc+2, #low  ConfigDesc
    mov   rLen,    #32                  ;sizeof(ConfigDesc)
    sjmp  Setup_break
GetDescUnknown:
    setb  bRequestError
    sjmp  Setup_break
Set_Address:
    sjmp  Set_Config
Get_Config:
    mov   DPTR, # EP0_BUFFER+0
    mov   A, wValueLo
    movx  @DPTR, A
    mov   rLen, #1
    sjmp  Setup_break
Set_Config:                             ;
    mov   DPTR, # EP0_BUFFER+2
    movx  A, @DPTR
    mov   wValueLo, A
    sjmp  Setup_break
Setup_default:
    setb  bRequestError
    sjmp  Setup_break

code_3BF3:                              ;
    setb  bRequestError
Setup_break:
    jnb   bRequestError, code_3C00
    mov   bRequest,  #-1
    mov   UEP0_CTRL, #bUEP_R_TOG or bUEP_T_TOG or UEP_R_RES_STALL or UEP_T_RES_STALL
    sjmp  break

code_3C00:                              ;
    mov   A, reqLen
    setb  C
    subb  A, rLen
    jc    code_3C0A
    mov   reqLen, rLen
code_3C0A:                              ;
    Xcall PreparePacket
    mov   UEP0_CTRL, #bUEP_R_TOG or bUEP_T_TOG
    sjmp  break

EP0InTransfer:                          ;
    mov   A, bRequest
    add   A, #0FBh                      ;
    jz    code_3C23
    dec   A
    jnz   code_3C2D
    Xcall PreparePacket
    xrl   UEP0_CTRL, # bUEP_T_TOG       ;
    sjmp  break

code_3C23:                              ;USB_DEV_AD = (USB_DEV_AD & bUDA_GP_BIT) < wValueLo
    mov   A, USB_DEV_AD                 ;
    anl   A, #bUDA_GP_BIT
    orl   A, wValueLo
    mov   USB_DEV_AD, A                 ;
    sjmp  EP0OutTransfer

code_3C2D:                              ;
    clr   A
    mov   UEP0_T_LEN, A                 ;
    sjmp  EP0OutTransfer

EP0OutTransfer:                         ;
    mov   UEP0_CTRL, #UEP_T_RES_NAK     ;
break:                                  ;
    clr   UIF_TRANSFER                  ;
    ret

UsbReset:                               ;
    jnb   UIF_BUS_RST, UsbSuspend       ;
    mov   UEP0_CTRL, # UEP_T_RES_NAK    ;
    mov   UEP1_CTRL, # bUEP_AUTO_TOG    ; todo remove this
    mov   UEP2_CTRL, # bUEP_AUTO_TOG or UEP_T_RES_NAK
    clr   A
    mov   USB_DEV_AD, A                 ;
    clr   UIF_SUSPEND                   ;
    clr   UIF_TRANSFER                  ;
    clr   UIF_BUS_RST                   ;
    ret
UsbSuspend:                             ;3C4E
    jnb   UIF_SUSPEND,  UsbDone         ;
    clr   UIF_SUSPEND                   ;
    ret
UsbDone:                                ;3C54
    mov   USB_INT_FG, #0FFh             ;
    ret

; =============== S U B R O U T I N E =======================================
;void main(void)
Main:                                   ;3C58
if (REPLACE_KEY_INIT = 1)
    Xcall InitScramble                  
endif
    mov   SAFE_MOD, # 055h              ;
    mov   SAFE_MOD, # 0AAh              ;
    mov   A, CLOCK_CFG                  ;
    anl   A, #not MASK_SYS_CK_SEL       ;
    orl   A, #4                         ;12MHz
    mov   CLOCK_CFG, A                  ;
    clr   A
    mov   SAFE_MOD, A                   ;
if (REMOVE_SERIAL_CODE = 0)
    clr   bSerialMode                   ;
endif
    clr   bRunMode                      ;
    setb  bWriteProtected               ;    
    clr   bResponsValid                 ; 
    clr   EA                            ; 
    clr   TR0                           ;
    clr   TF0                           ;
if (REMOVE_SERIAL_CODE = 0)
    clr   U1SM0                         ;
    setb  U1SMOD                        ;
    setb  U1REN                         ;
    mov   SBAUD1, #0F3h                 ;57600 @ 12MHz
    clr   U1TI                          ;
    clr   U1RI                          ;
endif
;   if (!(GLOBAL_CFG & bBOOT_LOAD) || (IsProgButtonPressed()==1))
    mov   A, GLOBAL_CFG                 ;if (!(GLOBAL_CFG & bBOOT_LOAD) ||
    jnb   ACC.5, code_3C91              ;
    Xcall IsProgButtonPressed           ;     (IsProgButtonPressed()==1))
    mov   A, R7                         ;  
    xrl   A, #1
    jnz   code_3CBC
;enable UsbCore
code_3C91:                              ;{
    clr   A
    mov   USB_CTRL, A                   ;     init the usbcore
    mov   UEP2_3_MOD, #bUEP2_RX_EN or bUEP2_TX_EN;
    mov   UEP0_DMA_H, #high EP0_BUFFER  ;
    mov   UEP0_DMA_L, #low  EP0_BUFFER  ;
    mov   UEP2_DMA_H, #high EP2_OUT_BUFFER  ;
    mov   UEP2_DMA_L, #low  EP2_OUT_BUFFER  ;
    mov   UEP0_CTRL,  #UEP_T_RES_NAK    ;
    mov   UEP2_CTRL,  #bUEP_AUTO_TOG or UEP_T_RES_NAK;
    mov   USB_DEV_AD, A                 ;
    mov   UDEV_CTRL,  #bUD_PD_DIS       ;
    mov   USB_CTRL,   #bUC_DEV_PU_EN or bUC_INT_BUSY or bUC_DMA_EN;
    orl   UDEV_CTRL,  #bUD_PORT_EN      ;
    mov   USB_INT_FG, #0FFh             ;}
    mov   USB_INT_EN, #bUIE_SUSPEND or bUIE_TRANSFER or bUIE_BUS_RST
    sjmp  Mainloop

code_3CBC:                              ;else
    orl   TMOD, #1                      ;{
    mov   TL0,  #low  64C0h             ;   Timeout 40 ms @ 12 MHz
    mov   TH0,  #high 64C0h             ; 
    clr   TF0                           ;
    setb  TR0                           ;}
Mainloop:                               ;while(1) {
if (REMOVE_SERIAL_CODE = 0)
    jnb   U1RI, code_3CCF               ;  if (U1RI) HandleSerialEvents
    Xcall HandleSerialEvents
endif
code_3CCF:                              ;  if(USB_INT_FG & 0x07) HandleUsbEvents
    mov   A, USB_INT_FG                 ;
    anl   A, #7
    jz    code_3CD8
    Xcall HandleUsbEvents
code_3CD8:                              ;
    jnb   TF0, code_3CE4                ;   if(TF0) { //timeout
    clr   TF0                           ;
    clr   TR0                           ;
if (REMOVE_SERIAL_CODE = 0)
    jb    bSerialMode, code_3CE4        ;
endif
    setb  bRunMode

code_3CE4:                              ;
    jnb   bRunMode, Mainloop            ;   if (bRunMode)
    mov   DPTR, # (ROM_CFG_ADDR-4)      ;   {
    clr   A                             ;      if (*ROM_CFG_ADDR-4==0x57A8)   
    movc  A, @A+DPTR
    xrl   A, # 057h
    jnz   SoftReset
    inc   DPTR
    movc  A, @A+DPTR
    xrl   A, # 0A8h
    jnz   SoftReset                     ;      {
    mov   SAFE_MOD,  # 055h             ;         SAFE_MOD  = 0x55;
    mov   SAFE_MOD,  # 0AAh             ;         SAFE_MOD  = 0xAA;    
    mov   CLOCK_CFG, # bOSC_EN_INT or 3 ;         CLOCK_CFG = bOSC_EN_INT | 3;// 6MHz
    mov   SAFE_MOD,  A                  ;         SAFE_MOD  = 0;
    mov   USB_CTRL,  # bUC_RESET_SIE or bUC_CLR_ALL
    clr   U1SMOD                        ;         U1SMOD    = 0;
    clr   U1REN                         ;         U1REN     = 0;
    clr   TR0                           ;         TR0       = 0;
    mov   DPTR, # (ROM_CFG_ADDR-1)      ;          
    movc  A, @A+DPTR
    mov   R6, A                         ;         addr = CBYTE(0x3FF7);  //msb
    mov   DPTR, # (ROM_CFG_ADDR-2)      ;         addr = addr << 8 | CBYTE(0x3FF6);
    clr   A
    movc  A, @A+DPTR
    mov   R5, A                         ;         CALL(addr); 
    mov   A, R5
    xch   A, R2
    mov   A, R6
    xch   A, R2
    mov   R1, A
    mov   R3, #0
    mov   pCall+0, R3
    mov   pCall+1, R2
    mov   pCall+2, A
    Xcall C_Icall                       ;      }
SoftReset:                              ;   }
    mov   SAFE_MOD,   #055h             ;   SAFE_MOD = 0x55;
    mov   SAFE_MOD,   #0AAh             ;   SAFE_MOD = 0xAA;
    orl   GLOBAL_CFG, #bSW_RESET        ;   GLOBAL_CFG |=bSW_RESET;
    sjmp  $                             ;   while(1);

;******************************;
;from startup.a51
C_C51STARTUP:                  ;
    mov   R0, #127             ;
    clr   A
Idataloop:
    mov   @R0, A
    djnz  R0, Idataloop
    mov   SP, #Stack-1         ;
if (REPLACE_KEY_INIT = 1)
    Xjmp  Main
else    
    Xjmp  c_init

code_3D3C:
    Xjmp  Main
 
code_3D3F:
    clr   A
    movc  A, @A+DPTR
    inc   DPTR
    mov   R0, A

code_3D43:
    clr   A
    movc  A, @A+DPTR
    inc   DPTR
    jc    code_3D4B
    mov   @R0, A
    sjmp  code_3D4C

code_3D4B:
    movx  @R0, A
code_3D4C:
    inc   R0
    djnz  R7, code_3D43
    sjmp  code_3D7A

code_3D51:
    clr   A
    movc  A, @A+DPTR
    inc   DPTR
    mov   R0, A
    anl   A, #7
    add   A, #0Ch
    xch   A, R0
    clr   C
    rlc   A
    swap  A
    anl   A, #0Fh
    orl   A, #20h
    xch   A, R0
    movc  A, @A+PC
    jc    code_3D69
    cpl   A
    anl   A, @R0
    sjmp  code_3D6A
code_3D69:
    orl   A, @R0                        
code_3D6A:
    mov   @R0, A
    djnz  R7, code_3D51
    sjmp  code_3D7A

    db     01h
    db     02h
    db     04h
    db     08h
    db     10h
    db     20h
    db     40h
    db     80h

c_init:                                 ;
    mov   DPTR, # InitScramble                 
code_3D7A:                              ;
    clr   A
    mov   R6, #1
    movc  A, @A+DPTR
    jz    code_3D3C
    inc   DPTR
    mov   R7, A
    anl   A, #03Fh
    jnb   ACC.5, code_3D90                ;
    anl   A, #01Fh
    mov   R6, A
    clr   A
    movc  A, @A+DPTR
    inc   DPTR
    jz    code_3D90
    inc   R6
code_3D90:                              ;
    xch   A, R7                         
    anl   A, #0C0h                      
    add   A, ACC                        ;
    jz    code_3D3F                     
    jc    code_3D51                     
    clr   A                             
    movc  A, @A+DPTR                    
    inc   DPTR                          
    mov   R2, A                         
    clr   A                             
    movc  A, @A+DPTR                    
    inc   DPTR                          
    mov   R0, A                         
code_3DA1:                              ;
    clr   A                             
    movc  A, @A+DPTR                    
    inc   DPTR                          
    xch   A, R0                         
    xch   A, DPL                        ;
    xch   A, R0                         
    xch   A, R2                         
    xch   A, DPH                        ;
    xch   A, R2                         
    movx  @DPTR, A                      
    inc   DPTR                          
    xch   A, R0                         
    xch   A, DPL                        ;
    xch   A, R0                         
    xch   A, R2                         
    xch   A, DPH                        ;
    xch   A, R2
    djnz  R7, code_3DA1
    djnz  R6, code_3DA1
    sjmp  code_3D7A
endif
; =============== S U B R O U T I N E =======================================
if (REMOVE_SERIAL_CODE = 0)
HandleSerialEvents:                    ;
__chk     set R6
__len     set R5
__i       set R4

    jnb   U1RI, HandleSerial_done       ;if (!U1RI) return 
    Xcall Serial1GetChar                ;
    mov   A, R7                         ;if (Serial1GetChar()!=0x61) return;
    xrl   A, #061h
    jnz   HandleSerial_done
    Xcall Serial1GetChar                ;if (Serial1GetChar()!=0x9E) return;
    mov   A, R7
    xrl   A, #09Eh
    jnz   HandleSerial_done
    Xcall Serial1GetChar
    mov   R0, #cmdbuffer+0              ;cmdbuffer[0]=Serial1GetChar();
    mov   A, R7
    mov   @R0, A
    mov   __chk, A                      ;chk=cmdbuffer[0];
    Xcall Serial1GetChar                ;cmdbuffer[0]=Serial1GetChar();
    inc   R0                            ;  
    mov   A, R7
    mov   @R0, A
    mov   __len, A                      ;len=cmdbuffer[1]
    mov   A, @R0
    add   A, __chk                      ;chk+=cmdbuffer[1]
    mov   __chk, A
    dec   R0
    mov   A, @R0                        ;if(cmdbuffer[0]==0xA8 |)
    xrl   A, #0A8h
    jz    code_3DF0
    mov   A, @R0                        ;  | cmdbuffer[0]==0xA7
    xrl   A, #0A7h
    jz    code_3DF0
    mov   A, @R0                        ;  | cmdbuffer[0]==0xB6
    cjne  A, #0B6h, code_3DF2           ;
code_3DF0:                              ;
    inc   __len                         ;   len+=2
    inc   __len
code_3DF2:                              ;
    clr   A                             ;i=0
    mov   __i, A                        ;
SerRecieveLoop:                         ;3DF4:
    mov   A, __i                        ;while (i < len)
    xrl   A, __len
    jz    SerLoop_break                 ;{
    Xcall Serial1GetChar
    mov   A, #cmdbuffer+2               ;   cmdbuffer[2+i] = Serial1GetChar
    add   A, __i
    mov   R0, A
    mov   A, R7
    mov   @R0, A
    mov   A, #cmdbuffer+2               ;   chk+=cmdbuffer[2+i]
    add   A, __i
    mov   R0, A
    mov   A, @R0
    add   A, __chk
    mov   __chk, A
    inc   __i
    sjmp  SerRecieveLoop                ;}

SerLoop_break:                          ;
    Xcall Serial1GetChar                ;if (Serial1GetChar != chk)
    mov   A, __chk                      ;{
    xrl   A, R7
    jz    code_3E1C
    mov   R7, #055h                     ;  Serial1PutChar(0x55)
    Xcall Serial1PutChar
    mov   R7, #0AAh                     ;  Serial1PutChar(0xAA)
    Xjmp  Serial1PutChar                ;}
code_3E1C:                              ;
    setb  bSerialMode                   ;bSerialMode=1;
    Xcall Functiondispatcher            ;Functiondispatcher
HandleSerial_done:                      ;
    ret
endif
; =============== S U B R O U T I N E =======================================
_loadPtr:                               ;
    cjne  R3, #1, code_3E2B
    mov   DPL, R1                       ;
    mov   DPH, R2                       ;
    movx  A, @DPTR
    ret
code_3E2B:                              ;
    jnc   code_3E2F
    mov   A, @R1
    ret
code_3E2F:                              ;
    cjne  R3, # 0FEh, code_3E34         ; 
    movx  A, @R1
    ret
code_3E34:                              ;
    mov   DPL, R1                       ;
    mov   DPH, R2                       ;
    clr   A
    movc  A, @A+DPTR
    ret
; =============== S U B R O U T I N E =======================================
_storePtr:                             ;
    cjne  R3, #1, code_3E44
    mov   DPL, R1                      ;
    mov   DPH, R2                      ;
    movx  @DPTR, A
    ret
code_3E44:                             ;
    jnc   code_3E48
    mov   @R1, A
    ret
code_3E48:                             ;
    cjne  R3, # 0FEh, code_3E4C        ;
    movx  @R1, A
code_3E4C:                             ;
    ret
; =============== S U B R O U T I N E =======================================
_cc_case:      ;
    pop   DPH   ; 
    pop   DPL   ; 
    mov   R0, A
code_3E52:        ;
    clr   A
    movc  A, @A+DPTR
    jnz   code_3E68
    mov   A, #1
    movc  A, @A+DPTR
    jnz   code_3E68
    inc   DPTR
    inc   DPTR
code_3E5D:        ;
    movc  A, @A+DPTR
    mov   R0, A
    mov   A, #1
    movc  A, @A+DPTR
    mov   DPL, A    ; 
    mov   DPH, R0   ; 
    clr   A
    jmp   @A+DPTR
code_3E68:        ;
    mov   A, #2
    movc  A, @A+DPTR
    xrl   A, R0
    jz    code_3E5D
    inc   DPTR
    inc   DPTR
    inc   DPTR
    sjmp  code_3E52

; =============== S U B R O U T I N E =======================================
C_icall:
    mov   DPH, R2   ; Data Pointer, High Byte
    mov   DPL, R1   ; Data Pointer, Low Byte
    clr   A
    jmp   @A+DPTR


CopyRightMsg: db "USB DBG CH559 & ISP",0 ;

DeviceDesc:
    db     12h
    db     1
    db     10h,01h      ;bdcusb
    db     0FFh         ;class
    db     080h         ;subclass 
    db     055h         ;protocol
    db     8
    db     048h,043h
    db     0E0h,055h
if (CHANGE_BCD_VER = 0)
    db     00h,01h      ;bcddevice V1.0  
else
  if (USE_LOADER_EXT = 0)
    db     10h,01h      ;bcddevice V1.1  
  else
    db     11h,01h      ;bcddevice V1.11 
  endif
endif
    db     0            ;no strings
    db     0
    db     0
    db     1

ConfigDesc:
    db     9
    db     2
    db     20h,00h      ;totallength 0x0020
    db     1
    db     1
    db     0
    db     80h          ;
    db     32h          ;100 mA

    db     9
    db     4
    db     0
    db     0
    db     2
    db     0FFh
    db     080h 
    db     055h 
    db     0

    db     7
    db     5
    db     82h         ;EP2In
    db     2
    db     40h,00h     ;size 0x0040
    db     0

    db     7
    db     5
    db     2           ;EP2Out 
    db     2
    db     40h,00h     ;size 0x0040
    db     0

; =============== S U B R O U T I N E =======================================
PreparePacket:        ;
    mov   A, reqLen
    jz    code_3EF8
    clr   C
    subb  A, #8
    jc    code_3ECC
    mov   R7, #8
    sjmp  code_3ECE
code_3ECC:        ;
    mov   R7, reqLen
code_3ECE:        ;
    xch   A, R6
    mov   A, R7
    xch   A, R6
    mov   RAM_1B+0, pDesc+0
    mov   RAM_1B+1, pDesc+1
    mov   RAM_1B+2, pDesc+2
    mov   RAM_1E, R6
    mov   R3, #1
    mov   R2, #0
    mov   R1, #0
    Xcall _memcpy
    clr   C
    mov   A, reqLen
    subb  A, R6
    mov   reqLen, A
    mov   A, R6
    add   A, pDesc+2
    mov   pDesc+2, A
    clr   A
    addc  A, pDesc+1
    mov   pDesc+1, A
    mov   UEP0_T_LEN, R6      ;
    ret
code_3EF8:                    ;
    clr   A
    mov   UEP0_T_LEN, A ;
    ret

; =============== S U B R O U T I N E =======================================
FlashCode:                            ;
    mov   A, @R0
    mov   R6, A
    dec   R0
    mov   A, @R0
    mov   R5, A
    mov   A, R6
    mov   R4, A
    mov   R6, #high (ROM_CFG_ADDR-8)   ;

FlashCodeEntry:                        ; 3F05
    clr   C
    mov   A, R6
    subb  A, #high BOOT_LOAD_ADDR      ;< 3800?'
    jc    code_3F1C
    mov   A, R7
    subb  A, #low  (ROM_CFG_ADDR-4)    ;LoaderOptions
    mov   A, R6
    subb  A, #high (ROM_CFG_ADDR-4)    ;
    jc    code_3F32
    setb  C
    mov   A, R7
    subb  A, #low  (ROM_CFG_ADDR)      ;
    mov   A, R6
    subb  A, #high (ROM_CFG_ADDR)      ;
    jnc   code_3F32
code_3F1C:                             ;
    mov   ROM_ADDR_H, R6               ;
    mov   ROM_ADDR_L, R7               ;
    mov   ROM_DATA_H, R4               ;
    mov   ROM_DATA_L, R5               ;
    mov   A, ROM_CTRL                  ;
    jnb   ACC.6, code_3F32             ;
    mov   ROM_CTRL, #ROM_CMD_WRITE     ;
    mov   A, ROM_CTRL                  ;
    xrl   A, #bROM_ADDR_OK
    mov   R7, A
    ret
code_3F32:                             ;
    mov R7, #bROM_ADDR_OK              ;
    ret
; =============== S U B R O U T I N E =======================================
_memcpy:                               ;
    mov   RAM_18+0, R3
    mov   RAM_18+1, R2
    mov   RAM_18+2, R1
code_3F3B:                             ;
    mov   R7, RAM_1E
    dec   RAM_1E
    mov   A, R7
    jz    code_3F68
    mov   R3, RAM_1B+0
    inc   RAM_1B+2
    mov   A,  RAM_1B+2
    mov   R2, RAM_1B+1
    jnz   code_3F4E
    inc   RAM_1B+1
code_3F4E:
    dec   A
    mov   R1, A
    Xcall _LoadPtr
    mov   R7, A
    mov   R3, RAM_18+0
    inc   RAM_18+2
    mov   A,  RAM_18+2
    mov   R2, RAM_18+1
    jnz   code_3F60
    inc   RAM_18+1
code_3F60:                             ;
    dec   A
    mov   R1, A
    mov   A, R7
    Xcall _storePtr
    sjmp  code_3F3B
code_3F68:                             ;
    ret
; =============== S U B R O U T I N E =======================================
IsProgButtonPressed:                   ;3F69
    clr   A
    mov   R6, A
code_3F6B:                             ;
    jb    UDP, code_3F7B               ;
    mov   R7, #1
    Xcall Shortdelay
    inc   R6
    mov   A, R6
    jnb   ACC.6, code_3F6B             ;
    mov   R7, #0
    ret
code_3F7B:                             ;
    mov   R7, #1
    Xcall Shortdelay
    jnb   UDP, code_3F86               ;
    mov   R7, #1
    ret
code_3F86:                             ;
    mov   R7, #0
    ret
; =============== S U B R O U T I N E =======================================
FlashData:                             ;
    mov   A, R7
    add   A, ACC                       ;
    mov   ROM_ADDR_L, A                ;
    mov   ROM_DATA_L, R5               ;
    mov   A, ROM_CTRL                  ;
    jnb   ACC.6, code_3F9E             ;
    mov   ROM_CTRL, #ROM_CMD_WRITE     ;
    mov   A, ROM_CTRL                  ;
    xrl   A, # bROM_ADDR_OK
    mov   R7, A
    ret
code_3F9E:
    mov   R7, # bROM_ADDR_OK ;
    ret

; =============== S U B R O U T I N E =======================================
Shortdelay:                            ;3FA1
    mov   A, R7
    rrc   A
    rrc   A
    anl   A, #03Fh
    mov   R7, A
code_3FA7:                             ;
    mov   A, R7
    jz    done
    inc   SAFE_MOD                     ;
    dec   R7
    sjmp  code_3FA7
done:                                  ;
    ret

; =============== S U B R O U T I N E =======================================
ToggleProgMode:                        ;
    mov   SAFE_MOD, #055h              ;
    mov   SAFE_MOD, #0AAh              ;
    xrl   GLOBAL_CFG, #bCODE_WE or bDATA_WE
    clr   A
    mov   SAFE_MOD, A                  ;
    ret

; =============== S U B R O U T I N E =======================================
if (REMOVE_SERIAL_CODE = 0)
Serial1PutChar:                        ;
    clr   U1TI                         ;
    mov   SBUF1, R7                    ;
    jnb   U1TI, $
    clr   U1TI                         ;
    ret

; =============== S U B R O U T I N E =======================================
Serial1GetChar:
    jnb   U1RI, $
    clr   U1RI                         ;
    mov   R7, SBUF1                    ;
    ret
endif
; =============== S U B R O U T I N E =======================================
if (REPLACE_KEY_INIT = 0)
InitScramble:
    db    4
    db    scramble
    db    'T'
    db    'w'
    db    'c'
    db    'h'
    db    0
else    
InitScramble:                           ;init scramble to 'Twch'
    mov   Scramble+0,#'T'
    mov   Scramble+1,#'w'
    mov   Scramble+2,#'c'
    mov   Scramble+3,#'h'
    ret
endif

;todo check for address overflows
;and place a error message if PC gets bigger than 0x7EE

$NOLIST
;fill the rest with 0xFF
;Attention this will not work if a address overflow occurs
rept ((StartAddress or  07EEh) - $)
    DB    0FFh 
endM
$list
      dw   0FFFFh
       
;     dw   0xFFFF
;     dw   0xFFFF
;037F4
;     dw   0xFFFF                       ;must contain 0x57 and 0xA8
;     dw   0xFFFF                       ;att. this one is lsb first 
;
;cseg at ROM_CFG_ADDR
;     db   0
;     db   0xD2
;     db   0x65  ;undocumented byte

;     db   0x56  ;my chip serial no
;     db   0x48
;     db   0x7B
;     db   0x48
;     db   0x43
end
